from util import load_generation_tasks, read_userinfo, read_usertask, get_valid_participants, load_expertise, load_nasatlx
import pandas as pd
import numpy as np
from calc_measures import calc_user_reliance_measures, UserPerformance, calc_TiA_scale, calc_ATI_scale
from scipy.stats import wilcoxon, kruskal, spearmanr, mannwhitneyu
from pingouin import ancova, anova
from statsmodels.stats.multicomp import pairwise_tukeyhsd
import csv
import matplotlib.pyplot as plt
import seaborn as sns

# section 6.5.3

def transform_user_task_dict(reserved_users, user_task_dict, userinfo_dict, task_ID_list):
	answer_type_dict = {
		"control": "control",
		"concept": "concept",
		"archie_target": "archiet",
		"archie": "archie",
		"archie_od": "archieo"
	}
	new_user_task_dict = {}
	for user in reserved_users:
		new_user_task_dict[user] = {}
		user_group = userinfo_dict[user]
		for task_id in task_ID_list:
			new_user_task_dict[user][(task_id, "base")] = user_task_dict[user][(task_id, "base")]
			new_user_task_dict[user][(task_id, "advice")] = user_task_dict[user][(task_id, answer_type_dict[user_group])]
	return new_user_task_dict

def post_hoc_comparison(data_list_1, data_list_2, name1, name2):
	print("Use pots-hoc analysis")
	threshold = 0.05 / 4
	flag = False
	statistic, pvalue = mannwhitneyu(data_list_1, data_list_2, alternative='greater')
	if pvalue < threshold:
		print("Alternative {} > {},".format(name1, name2), "pvalue %.4f"%pvalue, "statistic %.4f"%statistic)
		flag = True
	statistic, pvalue = mannwhitneyu(data_list_1, data_list_2, alternative='less')
	if pvalue < threshold:
		print("Alternative {} < {},".format(name1, name2), "pvalue %.4f"%pvalue, "statistic %.4f"%statistic)
		flag = True
	if not flag:
		print("No significant difference with post-hoc analysis")

def get_all_reliance_measures(valid_users, userinfo_dict, new_user_task_dict, answer_dict, task_ID_list):
	num_tasks = 14.0
	performance_groups = {}
	performance_groups["control"] = []
	performance_groups["concept"] = []
	performance_groups["archie_target"] = []
	performance_groups["archie"] = []
	performance_groups["archie_od"] = []
	user2performance = {}
	for user in valid_users:
		tp_performance = UserPerformance(username=user)
		user_group = userinfo_dict[user]

		tp_correct, tp_agreement_fraction, tp_switching_fraction, tp_appropriate_reliance, initial_disagreement, relative_positive_ai_reliance, relative_positive_self_reliance = \
		calc_user_reliance_measures(user, usertask_dict=new_user_task_dict, answer_dict=answer_dict, task_id_list=task_ID_list)
		tp_accuracy = tp_correct / num_tasks

		tp_performance.add_performance(accuracy=tp_accuracy, agreement_fraction=tp_agreement_fraction, switching_fraction=tp_switching_fraction, 
			appropriate_reliance=tp_appropriate_reliance, relative_positive_ai_reliance=relative_positive_ai_reliance, 
			relative_positive_self_reliance=relative_positive_self_reliance, group="overall")
		user2performance[user] = tp_performance
		performance_groups[user_group].append(tp_performance)
	for key_ in performance_groups:
		print(key_, len(performance_groups[key_]))
	return user2performance

helpfulness_dict = {
	"unhelpful": -2,
	'somewhat unhelpful': -2,
	'neutral': -2,
	'somewhat helpful': 2,
	'helpful': 2
}

helpfulness_dict_ = {
	-2: "unhelpful",
	-1: 'somewhat unhelpful',
	0: 'neutral',
	1: 'somewhat helpful',
	2: 'helpful'
}

def load_opinion(input_file, column_name, userinfo_dict):
	df = pd.read_csv(input_file, usecols=['user_id', column_name], quoting=csv.QUOTE_MINIMAL)
	# , quoting=csv.QUOTE_NONE
	user_data_list = df.values.tolist()
	print(input_file, len(user_data_list))
	user_opinion_dict = {}
	helpfulness_list = []
	for user_id, opinion in user_data_list:
		if userinfo_dict[user_id] in ['archie', 'archie_od']:
			try:
				assert opinion.lower() != "none"
			except:
				print(user_id)
		try:
			if opinion.lower() == "none":
				continue
		except:
			print(user_id, opinion)
		try:
			user_opinion_dict[user_id] = helpfulness_dict[opinion]
		except:
			print(user_id)
		helpfulness_list.append(helpfulness_dict[opinion])
	return user_opinion_dict, helpfulness_list



if __name__ == '__main__':
	task_ID_list, data_dict = load_generation_tasks("selected_tasks_final.json")
	required_condtions = ["control", "concept", "archie_target", "archie", "archie_od"]
	user_task_dict, user_time_dict = read_usertask()
	userinfo_dict, user_task_order = read_userinfo(required_condtions=required_condtions)
	valid_users = get_valid_participants(user_task_dict, userinfo_dict, task_ID_list)
	# approved_participants = load_valid_prolific_ids()
	# print(f"{len(approved_participants)} participants get approved")
	# valid_users = valid_users & approved_participants
	print(f"In total, {len(valid_users)} available for analysis")

	new_user_task_dict = transform_user_task_dict(valid_users, user_task_dict, userinfo_dict, task_ID_list)
	answer_dict = {}
	for task_id in task_ID_list:
		answer_dict[task_id] = (data_dict[task_id].prediction, data_dict[task_id].ground_truth)
		# answer_dict[task_id] = (data_dict[task_id].ground_truth, data_dict[task_id].prediction)
	

	user2performance = get_all_reliance_measures(valid_users, userinfo_dict, new_user_task_dict, answer_dict, task_ID_list)
	user_ATI_scale, user_ATI_time = calc_ATI_scale()
	user_TiA_scale = calc_TiA_scale()
	user_expertise = load_expertise(valid_users)
	user_cognitive_load = load_nasatlx(valid_users)
	user_opinion_explanation, helpfulness_list = load_opinion("anonymized_data/PostQ_ham_2.csv", "opinion_explanation", userinfo_dict)
	
	TiA_subscales = ["Reliability/Competence", "Understanding/Predictability", "Familiarity", \
				 "Intention of Developers", "Propensity to Trust", "Trust in Automation"]
	variable_dict = {
		# "user_id": [],
		"ATI": [],
		# "TiA-Propensity": [],
		# "Tutorial": [],
		"opinion_explanation": [],
		"medical_expertise": [],
		"skin_cancer_expertise": [],
		"cognitive_load": [],
		"condition": [],
		"Accuracy": [],
		"Agreement_fraction": [],
		"switching_fraction": [],
		"Accuracy-wid": [],
		"RAIR": [],
		"RSR": []
	}
	for subscale in TiA_subscales:
		variable_dict[subscale] = []
	condtion_opinion_distribuiton = {}
	for condition in ["concept", "archie_target", "archie", "archie_od"]:
		condtion_opinion_distribuiton[condition] = {-2: 0, -1:0, 0:0, 1:0, 2:0}
		condtion_opinion_distribuiton[condition] = {-2: 0, 2:0}
	for user in valid_users:
		# variable_dict["user_id"].append(user)
		# variable_dict["TiA-Propensity"].append(user_ptt_scale[user])
		tp_condition = userinfo_dict[user]
		if tp_condition == "control":
			continue
		variable_dict["condition"].append(tp_condition)
		# print(user, tp_condition)
		variable_dict["opinion_explanation"].append(user_opinion_explanation[user])
		condtion_opinion_distribuiton[tp_condition][user_opinion_explanation[user]] += 1

		variable_dict["ATI"].append(user_ATI_scale[user])
		variable_dict["medical_expertise"].append(user_expertise[user]["medical_expertise"])
		variable_dict["skin_cancer_expertise"].append(user_expertise[user]["skin_cancer_expertise"])
		
		tp_performance = user2performance[user].performance

		# considering the performance of the second batch
		variable_dict["Accuracy"].append(tp_performance["accuracy"])
		variable_dict["Agreement_fraction"].append(tp_performance["agreement_fraction"])
		variable_dict["switching_fraction"].append(tp_performance["switching_fraction"])
		variable_dict["Accuracy-wid"].append(tp_performance["appropriate_reliance"])
		variable_dict["RAIR"].append(tp_performance["relative_positive_ai_reliance"])
		variable_dict["RSR"].append(tp_performance["relative_positive_self_reliance"])
		
		variable_dict["cognitive_load"].append(user_cognitive_load[user]['avg_cognitive_load'])

		# TiA subscales
		for subscale in TiA_subscales:
			variable_dict[subscale].append(user_TiA_scale[user][subscale])
	data_dict = {
		"condition": [],
		"ratio": [],
		"helpfulness": []
	}
	for condition in ["concept", "archie_target", "archie", "archie_od"]:
		print('-' * 17)
		print(condition)
		total_amount = sum([condtion_opinion_distribuiton[condition][index] for index in [-2, 2]])
		for index in range(-2, 3):
			if index not in [-2, 2]:
				continue
			data_dict["condition"].append(condition)
			data_dict["ratio"].append(condtion_opinion_distribuiton[condition][index] / total_amount)
			data_dict["helpfulness"].append(helpfulness_dict_[index].replace(" ", "\n"))
			print("%d\t%d\t%.3f"%(index, condtion_opinion_distribuiton[condition][index], condtion_opinion_distribuiton[condition][index] / total_amount))
	# df = pd.DataFrame(variable_dict)
	sns.barplot(data=data_dict, x="condition", y="ratio", hue="helpfulness")
	plt.show()
	print(len(variable_dict["opinion_explanation"]))
	for covariate_name in ["opinion_explanation"]:
		for subscale in ["Reliability/Competence", "Understanding/Predictability", "Intention of Developers", "Trust in Automation"]:
			# variable_name = "TiA-{}".format(subscale)
			correlation, pvalue = spearmanr(variable_dict[covariate_name], variable_dict[subscale])
			if pvalue < 0.05 / 4:
				print("Variable {} and variable {} have spearman correlation {:.3f} and pvalue {:.3f}".format(covariate_name, subscale, correlation, pvalue))
		print("-" * 17)
		correlation, pvalue = spearmanr(variable_dict[covariate_name], variable_dict["cognitive_load"])
		if pvalue < 0.05 / 4:
			print("Variable {} and variable {} have spearman correlation {:.3f} and pvalue {:.3f}".format(covariate_name, "cognitive_load", correlation, pvalue))

		for metric in  ["Accuracy", "Agreement_fraction", "switching_fraction", "Accuracy-wid", "RAIR", "RSR"]:
			correlation, pvalue = spearmanr(variable_dict[covariate_name], variable_dict[metric])
			# print(covariate_name, metric)
			print(len(variable_dict[covariate_name]), len(variable_dict[metric]))
			print(correlation, pvalue)
			if pvalue < 0.05 / 4:
				print("Variable {} and variable {} have spearman correlation {:.3f} and pvalue {:.3f}".format(covariate_name, metric, correlation, pvalue))
		print("-" * 34)












